<?php

namespace Drupal\dashboard_block_cards\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

class TotalTransactionsController extends ControllerBase {

  protected $currentUser;

  protected $entityTypeManager;

  protected $dateFormatter;

  protected $time;

  /**
   * @param \Drupal\Core\Session\AccountInterface $current_user
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(
    AccountInterface $current_user,
    EntityTypeManagerInterface $entity_type_manager,
    DateFormatterInterface $date_formatter,
    TimeInterface $time
  ) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->dateFormatter = $date_formatter;
    $this->time = $time;
  }

  /** {@inheritdoc} */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('date.formatter'),
      $container->get('datetime.time')
    );
  }

  public function getTotalTransactions(): JsonResponse {
    $user = $this->currentUser;

    if (!$user->isAuthenticated()) {
      return new JsonResponse(0);
    }

    $query = $this->entityTypeManager->getStorage('node')
      ->getQuery()
      ->condition('type', 'charging_history')
      ->condition('status', 1)
      ->condition('field_charger.entity.field_facility.entity.field_client.target_id', $this->currentUser->id())
      ->accessCheck(FALSE);


    $nids = $query->execute();

    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

    $total_transactions = count($nodes);

    return new JsonResponse($total_transactions);
  }

  public function getMonthlyTransactions(): JsonResponse {
    $user = $this->currentUser;

    if (!$user->isAuthenticated()) {
      return new JsonResponse(0);
    }

    $query = $this->entityTypeManager->getStorage('node')
      ->getQuery()
      ->condition('type', 'charging_history')
      ->condition('status', 1)
      ->condition('field_charger.entity.field_facility.entity.field_client.target_id', $this->currentUser->id())
      ->accessCheck(FALSE);

    $now = DrupalDateTime::createFromTimestamp($this->time->getRequestTime());

    $firstDayOfMonth = clone $now;
    $firstDayOfMonth->setTime(0, 0, 0);
    $firstDayOfMonth->setDate($now->format('Y'), $now->format('m'), 1);

    $lastDayOfMonth = clone $firstDayOfMonth;
    $lastDayOfMonth->modify('last day of this month');
    $lastDayOfMonth->setTime(23, 59, 59);

    $query->condition('field_date', $firstDayOfMonth->format('Y-m-d'), '>=');
    $query->condition('field_date', $lastDayOfMonth->format('Y-m-d'), '<=');

    $nids = $query->execute();

    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

    $total_transactions = count($nodes);

    return new JsonResponse($total_transactions);
  }
}
