<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Symfony\Component\HttpFoundation\Request;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;
use Drupal\Core\Mail\MailManagerInterface;

/**
 * Provides User Reset Password Resource
 *
 * @RestResource (
 *   id = "user_reset_password_resource",
 *   label = @Translation ("User Reset Password Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/reset-password"
 *   }
 * )
 */
class UserResetPasswordResource extends ResourceBase {

  /** @var EntityTypeManagerInterface */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**   * @var Client */
  protected Client $client;

  /** @var Request */
  protected Request $request;

  /** @var MailManagerInterface */
  protected MailManagerInterface $mailManager;

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Request $request,
    MailManagerInterface $mail_manager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->request = $request;
    $this->mailManager = $mail_manager;
  }

  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('plugin.manager.mail'),
    );
  }

  public function post($data): ResourceResponse {
    $email = $data['email'];
    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['mail' => $data['email']]);
    $user = reset($users);

    if (!$user) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'Email not found.',
      ], 404);
    }

    $new_password = $this->resetPassword($user);

    $params = [
      'new_password' => $new_password,
    ];

    $this->mailManager->mail(
      'user_rest_api',
      'reset_password_key',
      $email,
      'en',
      $params
    );

    return new ResourceResponse([
      'status' => 'Success',
      'message' => "Successful email sent at $email",
    ]);
  }

  public function resetPassword($user): string {
    $new_password = substr(
      str_shuffle('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()_+'),
      0,
      12
    );
    $user->setPassword($new_password);
    $user->save();

    return $new_password;
  }

  /** {@inheritdoc} */
  public function permissions(): array {
    return [];
  }

}
