<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides User Login Resource
 *
 * @RestResource (
 *   id = "user_set_balance_resource",
 *   label = @Translation ("User Set Balance Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/set-balance"
 *   }
 * )
 */
class UserSetBalanceResource extends ResourceBase {

  /** @var EntityTypeManagerInterface */
  protected EntityTypeManagerInterface $entityTypeManager;

  /** @var Client */
  protected Client $client;

  /** @var ConfigFactoryInterface */
  protected ConfigFactoryInterface $configFactory;

  /** @var AccountInterface */
  protected AccountInterface $currentUser;

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Client $client,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->client = $client;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
  }

  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  public function post($data): ResourceResponse {
    $user_id = $this->currentUser->id();
    $user = $this->entityTypeManager->getStorage('user')
      ->load($user_id);

    if (!$this->currentUser->isAuthenticated()) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'Unauthorized.',
      ], 401);
    }

    $user_balance = $user->field_customer_balance[0]->value;
    $new_balance = $user_balance + $data['amount'];
    $user->set('field_customer_balance', $new_balance);
    $user->save();

    $transaction = $this->setTransaction($user->id(), $data);

    return new ResourceResponse([
      'status' => 'Success',
      'message' => 'Successful set user balance.',
      'data' => [
        'user_id' => $user->id(),
        'transaction_id' => $transaction->field_transaction_id[0]->value,
        'transaction_amount' => $transaction->field_transaction_amount[0]->value,
      ],
    ]);
  }

  public function setTransaction($user_id, $data): EntityInterface {
    $node_data = [
      'type' => 'transaction_history',
      'title' => 'Transaction: ' . $data['transaction_id'],
      'field_user' => $user_id,
      'field_transaction_id' => $data['transaction_id'],
      'field_transaction_amount' => $data['amount'],
    ];

    $node = $this->entityTypeManager->getStorage('node')
      ->create($node_data);

    $node->save();

    return $node;
  }

  /** {@inheritdoc} */
  public function permissions(): array {
    return [];
  }

}
